#include <winver.h>
#include "../include/ec_version.h"

LANGUAGE            0x09,0x01
ETTERCAP_ICON ICON  "contrib/nsis/eNG.ico"

#define RC_VERSION  EC_VERSION_MAJOR, EC_VERSION_MINOR, EC_VERSION_REVISION, 0

VS_VERSION_INFO VERSIONINFO
  FILE_VERSION     RC_VERSION
  PRODUCTVERSION  RC_VERSION
  FILEFLAGSMASK   0x3FL
#ifdef DEBUG
  FILEFLAGS 1
#else
  FILEFLAGS 0
#endif
  FILEOS      VOS__WINDOWS32
  FILETYPE    VFT_DLL
  FILESUBTYPE 0x0L

BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904b0"
    BEGIN
      VALUE "CompanyName",      "The EtterCap community, http://ettercap.sourceforge.net/\0"
      VALUE "FileDescription",  "EtterCap sniffer\0"
      VALUE "FileVersion",      EC_VERSION "\0"
      VALUE "InternalName",     "ettercap-NG\0"
      VALUE "OriginalFilename", "ettercap.exe\0"
      VALUE "ProductName",      "The EtterCap sniffer\0"
      VALUE "ProductVersion",   EC_VERSION "\0"
      VALUE "LegalCopyright",   "Copyright " EC_COPYRIGHT " " EC_AUTHORS "\0"
    END
  END

  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x409, 1200
  END
END
